@@ -17,9 +17,8 @@
 #include "config.h"
 #include "intsym.h"
 
-/* For --nagging compilation with -std=c89, we need
-   to disable the inline keyword. */
-#ifdef PLAIN_C89
+/* Disable inline for non-C99 compilers. */
+#if !defined(__STDC_VERSION__) || __STDC_VERSION__ < 199901L
 #ifndef inline
 #define inline
 #endif
@@ -66,6 +65,9 @@
 #ifndef SIZE_MAX
 #define SIZE_MAX ((size_t)-1)
 #endif
+#ifndef SSIZE_MAX
+#define SSIZE_MAX ((size_t)-1/2)
+#endif
 #ifndef ULONG_MAX
 #define ULONG_MAX ((unsigned long)-1)
 #endif
@@ -101,8 +103,10 @@
 
 typedef unsigned char byte;
 
-#ifdef _MSC_VER
-typedef long ssize_t;
+#if defined(_MSC_VER) && !defined(MPG123_DEF_SSIZE_T)
+#define MPG123_DEF_SSIZE_T
+#include <stddef.h>
+typedef ptrdiff_t ssize_t;
 #endif
 
 /* A safe realloc also for very old systems where realloc(NULL, size) returns NULL. */
@@ -118,7 +122,7 @@ char* compat_strdup(const char *s);
 /* If we have the size checks enabled, try to derive some sane printfs.
    Simple start: Use max integer type and format if long is not big enough.
    I am hesitating to use %ll without making sure that it's there... */
-#if !(defined PLAIN_C89) && (defined SIZEOF_OFF_T) && (SIZEOF_OFF_T > SIZEOF_LONG) && (defined PRIiMAX)
+#if (defined SIZEOF_OFF_T) && (SIZEOF_OFF_T > SIZEOF_LONG) && (defined PRIiMAX)
 # define OFF_P PRIiMAX
 typedef intmax_t off_p;
 #else
@@ -126,7 +130,7 @@ typedef intmax_t off_p;
 typedef long off_p;
 #endif
 
-#if !(defined PLAIN_C89) && (defined SIZEOF_SIZE_T) && (SIZEOF_SIZE_T > SIZEOF_LONG) && (defined PRIuMAX)
+#if (defined SIZEOF_SIZE_T) && (SIZEOF_SIZE_T > SIZEOF_LONG) && (defined PRIuMAX)
 # define SIZE_P PRIuMAX
 typedef uintmax_t size_p;
 #else
@@ -134,7 +138,7 @@ typedef uintmax_t size_p;
 typedef unsigned long size_p;
 #endif
 
-#if !(defined PLAIN_C89) && (defined SIZEOF_SSIZE_T) && (SIZEOF_SSIZE_T > SIZEOF_LONG) && (defined PRIiMAX)
+#if (defined SIZEOF_SSIZE_T) && (SIZEOF_SSIZE_T > SIZEOF_LONG) && (defined PRIiMAX)
 # define SSIZE_P PRIuMAX
 typedef intmax_t ssize_p;
 #else
@@ -142,6 +146,10 @@ typedef intmax_t ssize_p;
 typedef long ssize_p;
 #endif
 
+/* Get an environment variable, possibly converted to UTF-8 from wide string.
+   The return value is a copy that you shall free. */
+char *compat_getenv(const char* name);
+
 /**
  * Opening a file handle can be different.
  * This function here is defined to take a path in native encoding (ISO8859 / UTF-8 / ...), or, when MS Windows Unicode support is enabled, an UTF-8 string that will be converted back to native UCS-2 (wide character) before calling the system's open function.
@@ -196,11 +204,73 @@ int win32_wide_utf8(const wchar_t * const wptr, char **mbptr, size_t * buflen);
 int win32_utf8_wide(const char *const mbptr, wchar_t **wptr, size_t *buflen);
 #endif
 
+/*
+	A little bit of path abstraction: We always work with plain char strings
+	that usually represent POSIX-ish UTF-8 paths (something like c:/some/file
+	might appear). For Windows, those are converted to wide strings with \
+	instead of / and possible fun is had with prefixes to get around the old
+	path length limit. Outside of the compat library, that stuff should not
+	matter, although something like //?/UNC/server/some/file could be thrown
+	around as UTF-8 string, to be converted to a wide \\?\UNC\server\some\file
+	just before handing it to Windows API.
+
+	There is a lot of unnecessary memory allocation and string copying because
+	of this, but this filesystem stuff is not really relevant to mpg123
+	performance, so the goal is to keep the code outside the compatibility layer
+	simple.
+*/
+
+/*
+	Concatenate a prefix and a path, one of them alowed to be NULL.
+	If the path is already absolute, the prefix is ignored. Relative
+	parts (like /..) are resolved if this is sensible for the platform
+	(meaning: for Windows), else they are preserved (on POSIX, actual
+	file system access would be needed because of symlinks).
+*/
+char* compat_catpath(const char *prefix, const char* path);
+
+/* Return 1 if the given path indicates an existing directory,
+   0 otherwise. */
+int compat_isdir(const char *path);
+
+/*
+	Directory traversal. This talks ASCII/UTF-8 paths externally, converts
+	to/from wchar_t internally if the platform wants that. Returning NULL
+	means failure to open/end of listing.
+	There is no promise about sorting entries.
+*/
+struct compat_dir;
+/* Returns NULL if either directory failed to open or listing is empty.
+   Listing can still be empty even if non-NULL, so always rely on the
+   nextfile/nextdir functions. */
+struct compat_dir* compat_diropen(char *path);
+void               compat_dirclose(struct compat_dir*);
+/* Get the next entry that is a file (or symlink to one).
+   The returned string is a copy that needs to be freed after use. */
+char* compat_nextfile(struct compat_dir*);
+/* Get the next entry that is a directory (or symlink to one).
+   The returned string is a copy that needs to be freed after use. */
+char* compat_nextdir (struct compat_dir*);
+
+#ifdef USE_MODULES
+/*
+	For keeping the path mess local, a system-specific dlopen() variant
+	is contained in here, too. This is very thin wrapping, even sparing
+	definition of a handle type, just using void pointers.
+	Use of absolute paths is a good idea if you want to be sure which
+	file is openend, as default search paths vary.
+*/
+void *compat_dlopen (const char *path);
+void *compat_dlsym  (void *handle, const char* name);
+void  compat_dlclose(void *handle);
+#endif
+
 /* Blocking write/read of data with signal resilience.
-   Both continue after being interrupted by signals and always return the
+   They continue after being interrupted by signals and always return the
    amount of processed data (shortage indicating actual problem or EOF). */
 size_t unintr_write(int fd, void const *buffer, size_t bytes);
 size_t unintr_read (int fd, void *buffer, size_t bytes);
+size_t unintr_fwrite(const void *ptr, size_t size, size_t nmemb, FILE *stream);
 
 /* That one comes from Tellie on OS/2, needed in resolver. */
 #ifdef __KLIBC__
