@@ -1,7 +1,7 @@
 /*
 	control_generic.c: control interface for frontends and real console warriors
 
-	copyright 1997-99,2004-8 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright 1997-99,2004-20 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written by Andreas Neuhaus and Michael Hipp
 	reworked by Thomas Orgis - it was the entry point for eventually becoming maintainer...
@@ -36,6 +36,7 @@
 #include "common.h"
 #include "genre.h"
 #include "playlist.h"
+#include "metaprint.h"
 #include "audio.h"
 #define MODE_STOPPED 0
 #define MODE_PLAYING 1
@@ -53,6 +54,7 @@ int control_file = STDIN_FILENO;
 #endif /* WANT_WIN32_FIFO */
 #endif
 FILE *outstream;
+int out_is_term = FALSE;
 static int mode = MODE_STOPPED;
 static int init = 0;
 
@@ -68,15 +70,69 @@ void generic_sendmsg (const char *fmt, ...)
 	fprintf(outstream, "\n");
 }
 
+// Variant that takes the first argument as a string subject to filtering.
+static void generic_sendstr(int is_utf8, const char *fmt, char* str, ...)
+{
+	va_list ap;
+	va_start(ap, str);
+	mpg123_string outbuf;
+	mpg123_init_string(&outbuf);
+	outstr(&outbuf, str, is_utf8, out_is_term);
+	generic_sendmsg(fmt, MPGSTR(outbuf), ap);
+	mpg123_free_string(&outbuf);
+	va_end(ap);
+}
+
+// Another one for two strings. I want to avoid writing a full format string parser.
+static void generic_send2str( int is_utf8, const char *fmt
+,	char* str, char *str2, ... )
+{
+	va_list ap;
+	va_start(ap, str2);
+	mpg123_string outbuf[2];
+	mpg123_init_string(outbuf);
+	mpg123_init_string(outbuf+1);
+	outstr(outbuf,   str,  is_utf8, out_is_term);
+	outstr(outbuf+1, str2, is_utf8, out_is_term);
+	generic_sendmsg(fmt, MPGSTR(outbuf[0]), MPGSTR(outbuf[1]), ap);
+	mpg123_free_string(outbuf+1);
+	mpg123_free_string(outbuf);
+	va_end(ap);
+}
+
+// Another one for three strings. Meh ...
+static void generic_send3str( int is_utf8, const char *fmt
+,	char* str, char *str2, char *str3, ... )
+{
+	va_list ap;
+	va_start(ap, str3);
+	mpg123_string outbuf[3];
+	mpg123_init_string(outbuf);
+	mpg123_init_string(outbuf+1);
+	mpg123_init_string(outbuf+2);
+	outstr(outbuf,   str,  is_utf8, out_is_term);
+	outstr(outbuf+1, str2, is_utf8, out_is_term);
+	outstr(outbuf+2, str3, is_utf8, out_is_term);
+	generic_sendmsg( fmt
+	,	MPGSTR(outbuf[0]), MPGSTR(outbuf[1]), MPGSTR(outbuf[2])
+	,	ap );
+	mpg123_free_string(outbuf+2);
+	mpg123_free_string(outbuf+1);
+	mpg123_free_string(outbuf);
+	va_end(ap);
+}
+
 /* Split up a number of lines separated by \n, \r, both or just zero byte
    and print out each line with specified prefix. */
-static void generic_send_lines(const char* fmt, mpg123_string *inlines)
+static void generic_send_lines(int is_utf8, const char* fmt, mpg123_string *inlines)
 {
 	size_t i;
 	int hadcr = 0, hadlf = 0;
 	char *lines = NULL;
 	char *line  = NULL;
 	size_t len = 0;
+	mpg123_string outbuf;
+	mpg123_init_string(&outbuf);
 
 	if(inlines != NULL && inlines->fill)
 	{
@@ -98,7 +154,8 @@ static void generic_send_lines(const char* fmt, mpg123_string *inlines)
 			if(line)
 			{
 				lines[i] = 0;
-				generic_sendmsg(fmt, line);
+				outstr(&outbuf, line, is_utf8, out_is_term);
+				generic_sendmsg(fmt, outbuf.fill ? outbuf.p : "???");
 				line = NULL;
 				lines[i] = save;
 			}
@@ -109,6 +166,7 @@ static void generic_send_lines(const char* fmt, mpg123_string *inlines)
 			if(line == NULL) line = lines+i;
 		}
 	}
+	mpg123_free_string(&outbuf);
 }
 
 void generic_sendstat (mpg123_handle *fr)
@@ -119,15 +177,30 @@ void generic_sendstat (mpg123_handle *fr)
 	generic_sendmsg("F %"OFF_P" %"OFF_P" %3.2f %3.2f", (off_p)current_frame, (off_p)frames_left, current_seconds, seconds_left);
 }
 
+// This is only valid as herlper to generic_sendv1, observe info memory usage!
+static void v1add(mpg123_string *buf, char *info, const char *str, size_t len)
+{
+	memset(info, 0, len);
+	if(!unknown2utf8(buf, str, len))
+	{
+		outstr(buf+1, buf[0].p, 1, out_is_term);
+		memcpy(info, buf[1].fill ? buf[1].p : "", buf[1].fill >=len ? len : buf[1].fill);
+	}
+}
+
 static void generic_sendv1(mpg123_id3v1 *v1, const char *prefix)
 {
 	int i;
 	char info[125] = "";
-	memcpy(info,    v1->title,   30);
-	memcpy(info+30, v1->artist,  30);
-	memcpy(info+60, v1->album,   30);
-	memcpy(info+90, v1->year,     4);
-	memcpy(info+94, v1->comment, 30);
+	mpg123_string buf[2];
+	mpg123_init_string(buf);
+	mpg123_init_string(buf+1);
+
+	v1add(buf, info,    v1->title,   30);
+	v1add(buf, info+30, v1->artist,  30);
+	v1add(buf, info+60, v1->album,   30);
+	v1add(buf, info+90, v1->year,     4);
+	v1add(buf, info+94, v1->comment, 30);
 
 	for(i=0;i<124; ++i) if(info[i] == 0) info[i] = ' ';
 	info[i] = 0;
@@ -135,6 +208,8 @@ static void generic_sendv1(mpg123_id3v1 *v1, const char *prefix)
 	generic_sendmsg("%s ID3.genre:%i", prefix, v1->genre);
 	if(v1->comment[28] == 0 && v1->comment[29] != 0)
 	generic_sendmsg("%s ID3.track:%i", prefix, (unsigned char)v1->comment[29]);
+	mpg123_free_string(buf+1);
+	mpg123_free_string(buf);
 }
 
 static void generic_sendinfoid3(mpg123_handle *mh)
@@ -152,12 +227,12 @@ static void generic_sendinfoid3(mpg123_handle *mh)
 	}
 	if(v2 != NULL)
 	{
-		generic_send_lines("I ID3v2.title:%s",   v2->title);
-		generic_send_lines("I ID3v2.artist:%s",  v2->artist);
-		generic_send_lines("I ID3v2.album:%s",   v2->album);
-		generic_send_lines("I ID3v2.year:%s",    v2->year);
-		generic_send_lines("I ID3v2.comment:%s", v2->comment);
-		generic_send_lines("I ID3v2.genre:%s",   v2->genre);
+		generic_send_lines(1, "I ID3v2.title:%s",   v2->title);
+		generic_send_lines(1, "I ID3v2.artist:%s",  v2->artist);
+		generic_send_lines(1, "I ID3v2.album:%s",   v2->album);
+		generic_send_lines(1, "I ID3v2.year:%s",    v2->year);
+		generic_send_lines(1, "I ID3v2.comment:%s", v2->comment);
+		generic_send_lines(1, "I ID3v2.genre:%s",   v2->genre);
 	}
 }
 
@@ -182,18 +257,17 @@ void generic_sendalltag(mpg123_handle *mh)
 			char id[5];
 			memcpy(id, v2->text[i].id, 4);
 			id[4] = 0;
-			generic_sendmsg("T ID3v2.%s:", id);
-			generic_send_lines("T =%s", &v2->text[i].text);
+			generic_sendstr(1, "T ID3v2.%s:", id);
+			generic_send_lines(1, "T =%s", &v2->text[i].text);
 		}
 		for(i=0; i<v2->extras; ++i)
 		{
 			char id[5];
 			memcpy(id, v2->extra[i].id, 4);
 			id[4] = 0;
-			generic_sendmsg("T ID3v2.%s desc(%s):",
-			        id,
-			        v2->extra[i].description.fill ? v2->extra[i].description.p : "" );
-			generic_send_lines("T =%s", &v2->extra[i].text);
+			generic_send2str( 1, "T ID3v2.%s desc(%s)"
+			,	id, MPGSTR(v2->extra[i].description) );
+			generic_send_lines(1, "T =%s", &v2->extra[i].text);
 		}
 		for(i=0; i<v2->comments; ++i)
 		{
@@ -203,10 +277,9 @@ void generic_sendalltag(mpg123_handle *mh)
 			id[4] = 0;
 			memcpy(lang, v2->comment_list[i].lang, 3);
 			lang[3] = 0;
-			generic_sendmsg("T ID3v2.%s lang(%s) desc(%s):",
-			                id, lang,
-			                v2->comment_list[i].description.fill ? v2->comment_list[i].description.p : "");
-			generic_send_lines("T =%s", &v2->comment_list[i].text);
+			generic_send3str( 1, "T ID3v2.%s lang(%s) desc(%s):",
+				id, lang, MPGSTR(v2->comment_list[i].description) );
+			generic_send_lines(1, "T =%s", &v2->comment_list[i].text);
 		}
 	}
 	generic_sendmsg("T }");
@@ -223,7 +296,7 @@ void generic_sendinfo (char *filename)
 	t = strrchr(s, '.');
 	if (t)
 		*t = 0;
-	generic_sendmsg("I %s", s);
+	generic_sendstr(0, "I %s", s);
 }
 
 static void generic_load(mpg123_handle *fr, char *arg, int state)
@@ -247,8 +320,8 @@ static void generic_load(mpg123_handle *fr, char *arg, int state)
 	}
 	else generic_sendinfo(arg);
 
-	if(htd.icy_name.fill) generic_sendmsg("I ICY-NAME: %s", htd.icy_name.p);
-	if(htd.icy_url.fill)  generic_sendmsg("I ICY-URL: %s", htd.icy_url.p);
+	if(htd.icy_name.fill) generic_sendstr(1, "I ICY-NAME: %s", htd.icy_name.p);
+	if(htd.icy_url.fill)  generic_sendstr(1, "I ICY-URL: %s",  htd.icy_url.p);
 
 	mode = state;
 	init = 1;
@@ -262,6 +335,8 @@ static void generic_loadlist(mpg123_handle *fr, char *arg)
 	long i = 0;
 	char *file = NULL;
 	char *thefile = NULL;
+	mpg123_string outbuf;
+	mpg123_init_string(&outbuf);
 
 	/* I feel retarted with string parsing outside Perl. */
 	while(*arg && isspace(*arg)) ++arg;
@@ -279,14 +354,16 @@ static void generic_loadlist(mpg123_handle *fr, char *arg)
 	param.listentry = 0; /* The playlist shall not filter. */
 	param.loop = 1;
 	param.shuffle = 0;
-	prepare_playlist(0, NULL);
+	int pl_utf8 = 0;
+	prepare_playlist(0, NULL, 0, &pl_utf8);
 	while((file = get_next_file()))
 	{
 		++i;
 		/* semantics: 0 brings you to the last track */
 		if(entry == 0 || entry == i) thefile = file;
 
-		generic_sendmsg("I LISTENTRY %li: %s", i, file);
+		outstr(&outbuf, file, pl_utf8, out_is_term);
+		generic_sendmsg("I LISTENTRY %li: %s", i, outbuf.fill ? outbuf.p : "???");
 	}
 	if(!i) generic_sendmsg("I LIST EMPTY");
 
@@ -294,6 +371,7 @@ static void generic_loadlist(mpg123_handle *fr, char *arg)
 	if(thefile) generic_load(fr, thefile, MODE_PLAYING);
 
 	free_playlist(); /* Free memory after it is not needed anymore. */
+	mpg123_free_string(&outbuf);
 }
 
 int control_generic (mpg123_handle *fr)
@@ -308,10 +386,15 @@ int control_generic (mpg123_handle *fr)
 
 	/* responses to stderr for frontends needing audio data from stdout */
 	if (param.remote_err)
- 		outstream = stderr;
- 	else
- 		outstream = stdout;
- 		
+	{
+		outstream = stderr;
+		out_is_term = stderr_is_term;
+	}
+	else
+	{
+		outstream = stdout;
+		out_is_term = stdout_is_term;
+	}
 #ifndef WIN32
  	setlinebuf(outstream);
 #else /* perhaps just use setvbuf as it's C89 */
@@ -394,7 +477,8 @@ int control_generic (mpg123_handle *fr)
 					{
 						char *meta;
 						if(mpg123_icy(fr, &meta) == MPG123_OK)
-						generic_sendmsg("I ICY-META: %s", meta != NULL ? meta : "<nil>");
+							generic_sendstr( 1, "I ICY-META: %s"
+							,	meta != NULL ? meta : "<nil>" );
 					}
 				}
 			}
@@ -411,13 +495,12 @@ int control_generic (mpg123_handle *fr)
 					break;
 			}
 		}
-
 		/*  on error */
-		if (n < 0) {
-			fprintf(stderr, "Error waiting for command: %s\n", strerror(errno));
+		if(n < 0)
+		{
+			merror("waiting for command: %s", strerror(errno));
 			return 1;
 		}
-
 		/* read & process commands */
 		if (n > 0)
 		{
@@ -513,6 +596,18 @@ int control_generic (mpg123_handle *fr)
 					continue;
 				}
 
+				if(!strcasecmp(comstr, "MUTE")) {
+					set_mute(ao, muted=TRUE);
+					generic_sendmsg("mute");
+					continue;
+				}
+
+				if(!strcasecmp(comstr, "UNMUTE")) {
+					set_mute(ao, muted=FALSE);
+					generic_sendmsg("unmute");
+					continue;
+				}
+
 				if(!strcasecmp(comstr, "T") || !strcasecmp(comstr, "TAG")) {
 					generic_sendalltag(fr);
 					continue;
@@ -546,7 +641,7 @@ int control_generic (mpg123_handle *fr)
 				{
 					long rate;
 					int ch;
-					int ret = mpg123_getformat(fr, &rate, &ch, NULL);
+					int ret = mpg123_getformat2(fr, &rate, &ch, NULL, 0);
 					/* I need to have portable printf specifiers that do not truncate the type... more autoconf... */
 					if(ret < 0) generic_sendmsg("E %s", mpg123_strerror(fr));
 					else generic_sendmsg("FORMAT %li %i", rate, ch);
@@ -594,6 +689,8 @@ int control_generic (mpg123_handle *fr)
 					generic_sendmsg("H STOP/S: stop playback (closes file)");
 					generic_sendmsg("H JUMP/J <frame>|<+offset>|<-offset>|<[+|-]seconds>s: jump to mpeg frame <frame> or change position by offset, same in seconds if number followed by \"s\"");
 					generic_sendmsg("H VOLUME/V <percent>: set volume in % (0..100...); float value");
+					generic_sendmsg("H MUTE: turn on software mute in output");
+					generic_sendmsg("H UNMUTE: turn off software mute in output");
 					generic_sendmsg("H RVA off|(mix|radio)|(album|audiophile): set rva mode");
 					generic_sendmsg("H EQ/E <channel> <band> <value>: set equalizer value for frequency band 0 to 31 on channel %i (left) or %i (right) or %i (both)", MPG123_LEFT, MPG123_RIGHT, MPG123_LR);
 					generic_sendmsg("H EQFILE <filename>: load EQ settings from a file");
@@ -790,9 +887,10 @@ int control_generic (mpg123_handle *fr)
 					if (!strcasecmp(cmd, "LP") || !strcasecmp(cmd, "LOADPAUSED")){ generic_load(fr, arg, MODE_PAUSED); continue; }
 
 					/* no command matched */
-					generic_sendmsg("E Unknown command: %s", cmd); /* unknown command */
+					generic_sendstr(0, "E Unknown command: %s", cmd);
 				} /* end commands with arguments */
-				else generic_sendmsg("E Unknown command or no arguments: %s", comstr); /* unknown command */
+				else generic_sendstr( 0, "E Unknown command or no arguments: %s"
+				,	comstr );
 
 				} /* end of single command processing */
 			} /* end of scanning the command buffer */
@@ -808,9 +906,19 @@ int control_generic (mpg123_handle *fr)
 			*/
 			if(buf[len-1] != 0)
 			{
-				char lasti = buf[len-1];
-				buf[len-1] = 0;
-				generic_sendmsg("E Unfinished command: %s%c", comstr, lasti);
+				// All that jazz because I did not reserve space for a zero.
+				char *last_command;
+				size_t last_len = len-(size_t)(next_comstr-buf);
+				last_command = malloc(last_len+1);
+				if(last_command)
+				{
+					memcpy(last_command, next_comstr, last_len);
+					last_command[last_len] = 0;
+					generic_sendstr(0, "E Unfinished command: %s", last_command);
+					free(last_command);
+				}
+				else
+					generic_sendmsg("E Unfinished command: <DOOM>");
 			}
 		} /* end command reading & processing */
 	} /* end main (alive) loop */
