@@ -84,6 +84,7 @@ struct mpg123_pars_struct
 	long feedpool;
 	long feedbuffer;
 #endif
+	long freeformat_framesize;
 };
 
 enum frame_state_flags
@@ -121,9 +122,10 @@ struct mpg123_handle_struct
 	float *decwin_mmx;
 	float *decwins;
 #endif
+#ifndef NO_EQUALIZER
 	int have_eq_settings;
 	real equalizer[2][32];
-
+#endif
 	/* for halfspeed mode */
 	unsigned char ssave[34];
 	int halfphase;
@@ -223,6 +225,7 @@ struct mpg123_handle_struct
 
 	/* bitstream info; bsi */
 	int bitindex;
+	long bits_avail;
 	unsigned char *wordpointer;
 	/* temporary storage for getbits stuff */
 	unsigned long ultmp;
@@ -247,7 +250,7 @@ struct mpg123_handle_struct
 	int fsizeold;
 	int ssize;
 	unsigned int bitreservoir;
-	unsigned char bsspace[2][MAXFRAMESIZE+512]; /* MAXFRAMESIZE */
+	unsigned char bsspace[2][MAXFRAMESIZE+512+4]; /* MAXFRAMESIZE */
 	unsigned char *bsbuf;
 	unsigned char *bsbufold;
 	int bsnum;
@@ -294,6 +297,8 @@ struct mpg123_handle_struct
 #ifndef NO_ID3V2
 	mpg123_id3v2 id3v2;
 #endif
+	unsigned char *id3v2_raw;
+	size_t id3v2_size;
 #ifndef NO_ICY
 	struct icy_meta icy;
 #endif
@@ -332,6 +337,11 @@ struct mpg123_handle_struct
 	void *wrapperdata;
 	/* A callback used to properly destruct the wrapper data. */
 	void (*wrapperclean)(void*);
+	int enc_delay;
+	int enc_padding;
+#ifndef NO_MOREINFO
+	struct mpg123_moreinfo *pinfo;
+#endif
 };
 
 /* generic init, does not include dynamic buffers */
@@ -375,9 +385,10 @@ MPEG 2.5
 576
 */
 
-#ifdef GAPLESS
-/* well, I take that one for granted... at least layer3 */
+// Well, I take that one for granted... at least layer3.
+// The value is needed for mpg123_getstate() in any build.
 #define GAPLESS_DELAY 529
+#ifdef GAPLESS
 void frame_gapless_init(mpg123_handle *fr, off_t framecount, off_t bskip, off_t eskip);
 void frame_gapless_realinit(mpg123_handle *fr);
 void frame_gapless_update(mpg123_handle *mh, off_t total_samples);
