@@ -1,7 +1,7 @@
 /*
 	lfs_wrap: Crappy wrapper code for supporting crappy ambiguous large file support.
 
-	copyright 2010 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright 2010-2020 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 
 	initially written by Thomas Orgis, thanks to Guido Draheim for consulting
@@ -467,6 +467,23 @@ int attribute_align_arg mpg123_position(mpg123_handle *mh, long frame_offset, lo
 	return MPG123_OK;
 }
 
+#undef mpg123_framelength
+/* off_t mpg123_framelength(mpg123_handle *mh); */
+long attribute_align_arg mpg123_framelength(mpg123_handle *mh)
+{
+	long val;
+	off_t largeval;
+
+	largeval = MPG123_LARGENAME(mpg123_framelength)(mh);
+	val = largeval;
+	if(val != largeval)
+	{
+		mh->err = MPG123_LFS_OVERFLOW;
+		return MPG123_ERR;
+	}
+	return val;
+}
+
 #undef mpg123_length
 /* off_t mpg123_length(mpg123_handle *mh); */
 long attribute_align_arg mpg123_length(mpg123_handle *mh)
@@ -535,7 +552,6 @@ off_t wrap_lseek(void *handle, off_t offset, int whence)
 			case IO_FD: return ioh->r_lseek(ioh->fd, smalloff, whence);
 			case IO_HANDLE: return ioh->r_h_lseek(ioh->handle, smalloff, whence);
 		}
-		error("Serious breakage - bad IO type in LFS wrapper!");
 		return -1;
 	}
 	else
@@ -554,6 +570,7 @@ off_t wrap_lseek(void *handle, off_t offset, int whence)
 #undef mpg123_replace_reader
 #undef mpg123_replace_reader_handle
 #undef mpg123_open
+#undef mpg123_open_fixed
 #undef mpg123_open_fd
 #undef mpg123_open_handle
 
@@ -682,6 +699,19 @@ int attribute_align_arg mpg123_open(mpg123_handle *mh, const char *path)
 	else return MPG123_LARGENAME(mpg123_open)(mh, path);
 }
 
+// This one needs to follow the logic of the original, and wrap the actual
+// mpg123_open() here.
+int attribute_align_arg mpg123_open_fixed( mpg123_handle *mh, const char *path
+,	int channels, int encoding )
+{
+	int err = open_fixed_pre(mh, channels, encoding);
+	if(err == MPG123_OK)
+		err = mpg123_open(mh, path);
+	if(err == MPG123_OK)
+		err = open_fixed_post(mh, channels, encoding);
+	return err;
+}
+
 /*
 	This is in fact very similar to the above:
 	The open routines always need to watch out for a prepared wrapper handle to use replaced normal I/O.
