@@ -16,7 +16,7 @@ int frame_bitrate(mpg123_handle *fr);
 long frame_freq(mpg123_handle *fr);
 int read_frame_recover(mpg123_handle* fr); /* dead? */
 int read_frame(mpg123_handle *fr);
-void set_pointer(mpg123_handle *fr, long backstep);
+void set_pointer(mpg123_handle *fr, int part2, long backstep);
 int position_info(mpg123_handle* fr, unsigned long no, long buffsize, unsigned long* frames_left, double* current_seconds, double* seconds_left);
 double compute_bpf(mpg123_handle *fr);
 long time_to_frame(mpg123_handle *fr, double seconds);
