@@ -1,13 +1,14 @@
 /*
 	audio: audio output interface
 
-	copyright ?-2016 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright ?-2020 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written by Michael Hipp
 */
 
 #include "out123_int.h"
 #include "wav.h"
+#include "hextxt.h"
 #ifndef NOXFERMEM
 #include "buffer.h"
 static int have_buffer(out123_handle *ao)
@@ -27,12 +28,12 @@ static int have_buffer(out123_handle *ao)
 
 static const char *default_name = "out123";
 
-static int modverbose(out123_handle *ao)
+static int modverbose(out123_handle *ao, int final)
 {
 	debug3("modverbose: %x %x %x"
 	,	(unsigned)ao->flags, (unsigned)ao->auxflags, (unsigned)OUT123_QUIET);
 	return AOQUIET
-	?	-1
+	?	(final ? 0 : -1)
 	:	ao->verbose;
 }
 
@@ -94,6 +95,7 @@ out123_handle* attribute_align_arg out123_new(void)
 	ao->channels = -1;
 	ao->format = -1;
 	ao->framesize = 0;
+	memset(ao->zerosample, 0, 8);
 	ao->state = play_dead;
 	ao->auxflags = 0;
 	ao->preload = 0.;
@@ -120,10 +122,16 @@ void attribute_align_arg out123_del(out123_handle *ao)
 	free(ao);
 }
 
+void attribute_align_arg out123_free(void *ptr)
+{
+	free(ptr);
+}
+
 /* Error reporting */
 
 /* Carefully keep that in sync with the error enum! */
-static const char *const errstring[OUT123_ERRCOUNT] =
+/* Sizing according to contents so that we can check! */
+static const char *const errstring[] =
 {
 	"no problem"
 ,	"out of memory"
@@ -135,6 +143,7 @@ static const char *const errstring[OUT123_ERRCOUNT] =
 ,	"failed to open device"
 ,	"buffer (communication) error"
 ,	"basic module system error"
+,	"bad function argument(s)"
 ,	"unknown parameter code"
 ,	"attempt to set read-only parameter"
 ,	"invalid out123 handle"
@@ -153,6 +162,8 @@ int out123_errcode(out123_handle *ao)
 
 const char* attribute_align_arg out123_plain_strerror(int errcode)
 {
+	if(errcode == OUT123_ERR)
+		return "some generic error";
 	if(errcode >= OUT123_ERRCOUNT || errcode < 0)
 		return "invalid error code";
 
@@ -210,6 +221,12 @@ out123_param( out123_handle *ao, enum out123_parms code
 		case OUT123_FLAGS:
 			ao->flags = (int)value;
 		break;
+		case OUT123_ADD_FLAGS:
+			ao->flags |= (int)value;
+		break;
+		case OUT123_REMOVE_FLAGS:
+			ao->flags &= ~((int)value);
+		break;
 		case OUT123_PRELOAD:
 			ao->preload = fvalue;
 		break;
@@ -269,6 +286,7 @@ out123_getparam( out123_handle *ao, enum out123_parms code
 	switch(code)
 	{
 		case OUT123_FLAGS:
+		case OUT123_ADD_FLAGS:
 			value = ao->flags;
 		break;
 		case OUT123_PRELOAD:
@@ -480,7 +498,7 @@ void attribute_align_arg out123_close(out123_handle *ao)
 		if(ao->deinit)
 			ao->deinit(ao);
 		if(ao->module)
-			close_module(ao->module, modverbose(ao));
+			close_module(ao->module, modverbose(ao, 0));
 		/* Null module methods and pointer. */
 		out123_clear_module(ao);
 	}
@@ -518,7 +536,16 @@ out123_start(out123_handle *ao, long rate, int channels, int encoding)
 	ao->rate      = rate;
 	ao->channels  = channels;
 	ao->format    = encoding;
-	ao->framesize = out123_encsize(encoding)*channels;
+	int samplesize = out123_encsize(encoding);
+	ao->framesize = samplesize*channels;
+	// The most convoluted way to say nothing at all.
+	for(int i=0; i<samplesize; ++i)
+#ifdef WORDS_BIGENDIAN
+		ao->zerosample[samplesize-1-i] =
+#else
+		ao->zerosample[i] =
+#endif
+		MPG123_ZEROSAMPLE(ao->format, samplesize, i);
 
 #ifndef NOXFERMEM
 	if(have_buffer(ao))
@@ -607,6 +634,30 @@ void attribute_align_arg out123_stop(out123_handle *ao)
 	ao->state = play_stopped;
 }
 
+// Replace the data in a given block of audio data with zeroes
+// in the correct encoding.
+static void mute_block( unsigned char *bytes, int count
+,	unsigned char* zerosample, int samplesize )
+{
+	// The count is expected to be a multiple of samplesize,
+	// this is just to ensure that the loop ends properly, should be noop.
+	count -= count % samplesize;
+	if(!count)
+		return;
+	// Initialize with one zero sample, then multiply that
+	// to eventually cover the whole buffer.
+	memcpy(bytes, zerosample, samplesize);
+	int offset = samplesize;
+	count     -= samplesize;
+	while(count)
+	{
+		int block = offset > count ? count : offset;
+		memcpy(bytes+offset, bytes, block);
+		offset += block;
+		count  -= block;
+	}
+}
+
 size_t attribute_align_arg
 out123_play(out123_handle *ao, void *bytes, size_t count)
 {
@@ -639,24 +690,41 @@ out123_play(out123_handle *ao, void *bytes, size_t count)
 		return buffer_write(ao, bytes, count);
 	else
 #endif
-	do /* Playback in a loop to be able to continue after interruptions. */
 	{
-		errno = 0;
-		written = ao->write(ao, (unsigned char*)bytes, (int)count);
-		debug4( "written: %d errno: %i (%s), keep_on=%d"
-		,	written, errno, strerror(errno)
-		,	ao->flags & OUT123_KEEP_PLAYING );
-		if(written >= 0){ sum+=written; count -= written; }
-		else if(errno != EINTR)
+		// Write 16K in a piece as maximum, as I've seen random short
+		// writes of big blocks with ALSA.
+		int maxcount = 1<<14;
+		maxcount -= maxcount % ao->framesize;
+		if(maxcount < 1)
+			maxcount = ao->framesize;
+		if(ao->flags & OUT123_MUTE)
+			mute_block( bytes, count, ao->zerosample
+			,	MPG123_SAMPLESIZE(ao->format) );
+		do /* Playback in a loop to be able to continue after interruptions. */
 		{
-			ao->errcode = OUT123_DEV_PLAY;
-			if(!AOQUIET)
-				error1("Error in writing audio (%s?)!", strerror(errno));
-			/* If written < 0, this is a serious issue ending this playback round. */
-			break;
-		}
-	} while(count && ao->flags & OUT123_KEEP_PLAYING);
-
+			errno = 0;
+			int block = count > maxcount ? maxcount : count;
+			written = ao->write(ao, bytes, block);
+			debug4( "written: %d errno: %i (%s), keep_on=%d"
+			,	written, errno, strerror(errno)
+			,	ao->flags & OUT123_KEEP_PLAYING );
+			if(written > 0)
+			{
+				bytes  = (char*)bytes+written;
+				sum   += written;
+				count -= written;
+			}
+			if(written < block && errno != EINTR)
+			{
+				ao->errcode = OUT123_DEV_PLAY;
+				if(!AOQUIET)
+					merror( "Error in writing audio, wrote only %d of %d (%s?)!"
+					,	written, block, strerror(errno) );
+				/* This is a serious issue ending this playback round. */
+				break;
+			}
+		} while(count && ao->flags & OUT123_KEEP_PLAYING);
+	}
 	debug3( "out123_play(%p, %p, ...) = %"SIZE_P
 	,	(void*)ao, bytes, (size_p)sum );
 	return sum;
@@ -822,6 +890,28 @@ static int open_fake_module(out123_handle *ao, const char *driver)
 		ao->drain = wav_drain;
 		ao->close = au_close;
 	}
+	else
+	if(!strcmp("hex", driver))
+	{
+		ao->propflags &= ~OUT123_PROP_LIVE;
+		ao->open  = hex_open;
+		ao->get_formats = hex_formats;
+		ao->write = hex_write;
+		ao->flush = builtin_nothing;
+		ao->drain = hextxt_drain;
+		ao->close = hextxt_close;
+	}
+	else
+	if(!strcmp("txt", driver))
+	{
+		ao->propflags &= ~OUT123_PROP_LIVE;
+		ao->open  = txt_open;
+		ao->get_formats = txt_formats;
+		ao->write = txt_write;
+		ao->flush = builtin_nothing;
+		ao->drain = hextxt_drain;
+		ao->close = hextxt_close;
+	}
 	else return OUT123_ERR;
 
 	return OUT123_OK;
@@ -844,13 +934,13 @@ static void check_output_module( out123_handle *ao
 		return;
 
 	/* Open the module, initial check for availability+libraries. */
-	ao->module = open_module( "output", name, modverbose(ao), ao->bindir);
+	ao->module = open_module( "output", name, modverbose(ao, final), ao->bindir);
 	if(!ao->module)
 		return;
 	/* Check if module supports output */
 	if(!ao->module->init_output)
 	{
-		if(final)
+		if(final && !AOQUIET)
 			error1("Module '%s' does not support audio output.", name);
 		goto check_output_module_cleanup;
 	}
@@ -885,7 +975,7 @@ static void check_output_module( out123_handle *ao
 
 check_output_module_cleanup:
 	/* Only if module did not check out we get to clean up here. */
-	close_module(ao->module, modverbose(ao));
+	close_module(ao->module, modverbose(ao, final));
 	out123_clear_module(ao);
 	return;
 }
@@ -918,7 +1008,7 @@ out123_drivers(out123_handle *ao, char ***names, char ***descr)
 	/* Wrap the call to isolate the lower levels from the user not being
 	   interested in both lists. it's a bit wasteful, but the code looks
 	   ugly enough already down there. */
-	count = list_modules("output", &tmpnames, &tmpdescr, modverbose(ao), ao->bindir);
+	count = list_modules("output", &tmpnames, &tmpdescr, modverbose(ao, 0), ao->bindir);
 	debug1("list_modules()=%i", count);
 	if(count < 0)
 	{
@@ -938,6 +1028,10 @@ out123_drivers(out123_handle *ao, char ***names, char ***descr)
 		,	"au", "Sun AU file (builtin)", &count )
 	||	stringlists_add( &tmpnames, &tmpdescr
 		,	"test", "output into the void (builtin)", &count )
+	||	stringlists_add( &tmpnames, &tmpdescr
+		,	"hex", "interleaved hex printout (builtin)", &count )
+	||	stringlists_add( &tmpnames, &tmpdescr
+		,	"txt", "plain text printout, a column per channel (builtin)", &count )
 	)
 		if(!AOQUIET)
 			error("OOM");
