@@ -1,7 +1,7 @@
 /*
 	alsa: sound output with Advanced Linux Sound Architecture 1.x API
 
-	copyright 2006-2016 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright 2006-2020 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 
 	initially written by Clemens Ladisch <clemens@ladisch.de>
@@ -14,6 +14,9 @@
    ... seems like the conclusion was not carried through.
  */
 #define _POSIX_SOURCE
+/* Things are still missing if _DEFAULT_SOURCE is not defined (for recent
+   glibc, I presume. */
+#define _DEFAULT_SOURCE
 #include "out123_int.h"
 #include <errno.h>
 
@@ -132,8 +135,8 @@ static int initialize_device(out123_handle *ao)
 		if(!AOQUIET) error("initialize_device(): cannot get sw params");
 		return -1;
 	}
-	/* start playing right away */
-	if (snd_pcm_sw_params_set_start_threshold(pcm, sw, 1) < 0) {
+	/* Start playing once we got at least full period ... this is not default? */
+	if (snd_pcm_sw_params_set_start_threshold(pcm, sw, buffer_size/2) < 0) {
 		if(!AOQUIET) error("initialize_device(): cannot set start threshold");
 		return -1;
 	}
@@ -235,7 +238,8 @@ static int write_alsa(out123_handle *ao, unsigned char *buf, int bytes)
 	}
 	if(written < 0)
 	{
-		error1("Fatal problem with alsa output, error %i.", (int)written);
+		if(!AOQUIET)
+			error1("Fatal problem with alsa output, error %i.", (int)written);
 		return -1;
 	}
 	else return snd_pcm_frames_to_bytes(pcm, written);
