@@ -1,11 +1,14 @@
 /*
 	id3dump: Print ID3 tags of files, scanned using libmpg123.
 
-	copyright 2007 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright 2007-2020 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written by Thomas Orgis
 */
 
+/* Need snprintf(). */
+#define _DEFAULT_SOURCE
+#define _BSD_SOURCE
 #include "config.h"
 #include "compat.h"
 #include "mpg123.h"
@@ -48,7 +51,7 @@ static void usage(int err)
 	fprintf(o,"\nNote that text output will always be in UTF-8, regardless of locale.\n");
 	exit(err);
 }
-static void want_usage(char* bla)
+static void want_usage(char* bla, topt *opts)
 {
 	usage(0);
 }
@@ -284,10 +287,13 @@ int open_picfile(const char* prefix, mpg123_picture* pic)
 	fd = compat_open(pfn, O_CREAT|O_WRONLY|O_EXCL);
 	while(fd < 0 && errno == EEXIST && ++count < ULONG_MAX)
 	{
-		char dum;
+		char dum[3];
 		size_t digits;
 
-		digits = snprintf(&dum, 1, "%lu", count);
+		// Just want to know the number of bytes needed.
+		// Modern compiler diagnostics complain if limit is smaller than
+		// format string, so increasing dummy to 3 characters.
+		digits = snprintf(dum, 3, "%lu", count);
 		if(!(pfn=safe_realloc(pfn, len+digits+1))) exit(11);
 
 		sprintf(pfn, "%s.%s%lu.%s", prefix, typestr, count, end);
@@ -321,10 +327,10 @@ static void store_pictures(const char* prefix, mpg123_id3v2 *v2)
 		fd = open_picfile(prefix, pic);
 		if(fd >= 0)
 		{ /* stream I/O for not having to care about interruptions */
-			FILE* picfile = fdopen(fd, "w");
+			FILE* picfile = compat_fdopen(fd, "w");
 			if(picfile)
 			{
-				if(fwrite(pic->data, pic->size, 1, picfile) != 1)
+				if(unintr_fwrite(pic->data, pic->size, 1, picfile) != 1)
 				{
 					error("Failure to write data.");
 					++errors;
